package com.ubuntu.builder;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.InputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.io.Serializable;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import javax.net.ssl.HttpsURLConnection;
import java.security.SignatureException;
import java.text.SimpleDateFormat;
import java.util.*;

import java.lang.*;
import java.lang.reflect.*;

import net.sf.json.JsonConfig;
import net.sf.json.JSONSerializer;
import net.sf.json.util.PropertyFilter;
import net.sf.json.util.JavaIdentifierTransformer;

public class Launchpad {

  public static class Branch {

    public Branch() {
    }
    
    public String branch_format;
    public String branch_type;
    public String bzr_identity;
    public String code_import_link;
    public String control_format;
    public String date_created;
    public String date_last_modified;
    public String dependent_branches_collection_link;
    public String description;
    public String display_name;
    public String explicitly_private;
    public String http_etag;
    public String landing_candidates_collection_link;
    public String landing_targets_collection_link;
    public String last_mirror_attempt;
    public String last_mirrored;
    public String last_scanned;
    public String last_scanned_id;
    public String lifecycle_status;
    public String linked_bugs_collection_link;
    public String merge_queue_config;
    public String merge_queue_link;
    public String mirror_status_message;
    public String name;
    public String owner_link;
    public String project_link;
    public String recipes_collection_link;
    public String registrant_link;
    public String repository_format;
    public String resource_type_link;
    public String reviewer_link;
    public String revision_count;
    public String self_link;
    public String sourcepackage_link;
    public String spec_links_collection_link;
    public String subscribers_collection_link;
    public String subscriptions_collection_link;
    public String unique_name;
    public String url;
    public String web_link;
    public String whiteboard;

    @Override public String toString() {
      StringBuilder result = new StringBuilder();
      String NEW_LINE = System.getProperty("line.separator");
      
      result.append(this.getClass().getName() + " Object {" + NEW_LINE);
      
      result.append("branch_format: " + branch_format + NEW_LINE);
      result.append("branch_type: " + branch_type + NEW_LINE);
      result.append("bzr_identity: " + bzr_identity + NEW_LINE);
      result.append("code_import_link: " + code_import_link + NEW_LINE);
      result.append("control_format: " + control_format + NEW_LINE);
      result.append("date_created: " + date_created + NEW_LINE);
      result.append("date_last_modified: " + date_last_modified + NEW_LINE);
      result.append("dependent_branches_collection_link: " + dependent_branches_collection_link + NEW_LINE);
      result.append("description: " + description + NEW_LINE);
      result.append("display_name: " + display_name + NEW_LINE);
      result.append("explicitly_private: " + explicitly_private + NEW_LINE);
      result.append("http_etag: " + http_etag + NEW_LINE);
      result.append("landing_candidates_collection_link: " +
                    landing_candidates_collection_link + NEW_LINE);
      result.append("landing_targets_collection_link: " +
                    landing_targets_collection_link + NEW_LINE);
      result.append("last_mirror_attempt: " + last_mirror_attempt + NEW_LINE);
      result.append("last_mirrored: " + last_mirrored + NEW_LINE);
      result.append("last_scanned: " + last_scanned + NEW_LINE);
      result.append("last_scanned_id: " + last_scanned_id + NEW_LINE);
      result.append("lifecycle_status: " + lifecycle_status + NEW_LINE);
      result.append("linked_bugs_collection_link: " +
                    linked_bugs_collection_link + NEW_LINE);
      result.append("merge_queue_config: " + merge_queue_config + NEW_LINE);
      result.append("merge_queue_link: " + merge_queue_link + NEW_LINE);
      result.append("mirror_status_message: " + mirror_status_message + NEW_LINE);
      result.append("name: " + name + NEW_LINE);
      result.append("owner_link: " + owner_link + NEW_LINE);
      //result.append( private;
      result.append("project_link: " + project_link + NEW_LINE);
      result.append("recipes_collection_link: " + recipes_collection_link + NEW_LINE);
      result.append("registrant_link: " + registrant_link + NEW_LINE);
      result.append("repository_format: " + repository_format + NEW_LINE);
      result.append("resource_type_link: " + resource_type_link + NEW_LINE);
      result.append("reviewer_link: " + reviewer_link + NEW_LINE);
      result.append("revision_count: " + revision_count + NEW_LINE);
      result.append("self_link: " + self_link + NEW_LINE);
      result.append("sourcepackage_link: " + sourcepackage_link + NEW_LINE);
      result.append("spec_links_collection_link: " + spec_links_collection_link + NEW_LINE);
      result.append("subscribers_collection_link: " + subscribers_collection_link + NEW_LINE);
      result.append("subscriptions_collection_link: " + subscriptions_collection_link + NEW_LINE);
      result.append("unique_name: " + unique_name + NEW_LINE);
      result.append("url: " + url + NEW_LINE);
      result.append("web_link: " + web_link + NEW_LINE);
      result.append("whiteboard: " + whiteboard + NEW_LINE);
      result.append("}");
      return result.toString();
    }
  }

  private static class PrivatePropertyFilter implements PropertyFilter {

    public boolean apply(Object source, String name, Object value) {
      return name.equals("private");
    }
    
  }
  
  public static Branch getBranch(String name) throws Exception {

    String uri = "https://api.launchpad.net/1.0/branches?ws.op=getByUniqueName&unique_name=" + name;
    
    URL url = new URL(uri);
    
    URLConnection conn = url.openConnection();
    InputStream in = conn.getInputStream();
    
    StringBuffer jsonBuffer = new StringBuffer();
    BufferedReader reader = new BufferedReader(new InputStreamReader(in));
    
    while(true) {      
      try {
        String line = reader.readLine();
        
        if(line == null)
          break;
        
        jsonBuffer.append(line);
      } catch(IOException e) {
        System.err.println(e);
        break;
      }
    }

    JsonConfig config = new JsonConfig();
    config.setJavaIdentifierTransformer(JavaIdentifierTransformer.UNDERSCORE);
    config.setJavaPropertyFilter(new PrivatePropertyFilter());
    config.setRootClass(Branch.class);
    Branch b = (Branch)JSONSerializer.toJava(
        JSONSerializer.toJSON(jsonBuffer.toString()),
        config);

    Branch branch = new Branch();
    Field[] fields = branch.getClass().getDeclaredFields();
    
    return branch;
  }
  
}